using EventGames;
using System;
using System.Runtime.InteropServices;
using UnityEngine;

public class JavaScriptCommunicator : ICommunicatable
{
    private enum State
    {
        None,
        EstablishingConnection,
        Running
    }
    private State state;

    private Action connectedCallback;
    private Action<string> messageReceivedCallback;

    [DllImport("__Internal")]
    private static extern int Out(string message);

    public void In(string message)
    {
        if (state == State.None)
        {
            return;
        }
        else if (state == State.EstablishingConnection)
        {
            state = State.Running;
            connectedCallback?.Invoke();
        }
        else if (state == State.Running)
        {
            if (message == "close")
            {
                state = State.None;
            }
            else
            {
                messageReceivedCallback?.Invoke(message);
            }
        }
    }

    public void Open(Action connectedCallback, Action<string> messageReceivedCallback)
    {
        new GameObject("JavaScriptCommunicatorReceiver").AddComponent<JavaScriptCommunicatorReceiver>().Setup(this);
        this.connectedCallback = connectedCallback;
        this.messageReceivedCallback = messageReceivedCallback;

        state = State.EstablishingConnection;
        Out("open");
    }
    public void Close()
    {
        state = State.None;
        Out("close");
    }

    public void Send(string message)
    {
        if (state == State.Running)
        {
            Out(message);
        }
    }

    public void Update()
    {

    }
}