#if UNITY_EDITOR
using UnityEditor;
using UnityEngine;
using UnityEngine.UIElements;

namespace EventGames
{
    public class SettingsWindow : SettingsProvider
    {
        private SerializedObject data;

        //here we tell Unity that the settings are at User scope and not Project
        public SettingsWindow(string path, SettingsScope scope = SettingsScope.User) : base(path, scope)
        {
        }

        //when the settings window is activated by Unity load settings data
        public override void OnActivate(string searchContext, VisualElement rootElement)
        {
            data = Settings.GetSerializedObject();
        }

        //here we tell Unity to setup a settings provider (settings window)
        [SettingsProvider]
        public static SettingsProvider CreateProvider()
        {
            return new SettingsWindow("Event Games", SettingsScope.User)
            {
                //makes it possible to find out settings when searching
                keywords = GetSearchKeywordsFromSerializedObject(Settings.GetSerializedObject())
            };
        }

        public override void OnGUI(string searchContext)
        {
            if (data == null ||
                data.targetObject == null ||
                EditorUtility.InstanceIDToObject(data.targetObject.GetInstanceID()) == null)
            {
                GUILayout.Label("missing data ScriptableObject");
            }
            else
            {
                //draw default ScriptableObject default inspector without the script field at the top.
                EditorGUI.BeginChangeCheck();

                data.Update();
                SerializedProperty Iterator = data.GetIterator();

                Iterator.NextVisible(true);
                while (Iterator.NextVisible(false))
                {
                    EditorGUILayout.PropertyField(Iterator, true);
                }

                data.ApplyModifiedProperties();
            }
        }
    }
}
#endif