using System;
using System.IO;
using UnityEngine;

namespace EventGames
{
    public class Settings : ScriptableObject
    {
        private static Settings _i;
        public static Settings I
        {
            get
            {
                if (_i == null)
                {
                    _i = GetScriptableObject();
                }
                return _i;
            }
        }

        private const string folderPath = "EventGames";
        private const string fileName = "NetworkLayer";

        [SerializeField]
        private bool debug;
        public bool Debug => debug;

        [SerializeField]
        private Communicators communicator;
        public Communicators Communicator => communicator;
        public enum Communicators
        {
            WebSocket,
            WebRTC,
            JavaScript
        }

        [SerializeField]
        private string screenId;
        public string ScreenId => screenId;

        [SerializeField]
        private string standaloneWebSocketServer;
        public string StandaloneWebSocketServer => standaloneWebSocketServer;

        [SerializeField]
        private TextAsset joystickSetup;
        public TextAsset JoystickSetup => joystickSetup;

        [SerializeField]
        private TextAsset joystickLocalized;
        public TextAsset JoystickLocalized => joystickLocalized;

        [SerializeField]
        private string[] colors;
        public string[] Colors => colors;

        [SerializeField]
        private string[] colorNames;
        public string[] ColorNames => colorNames;

        internal static Settings GetScriptableObject()
        {
            string resourcesFile = $"{folderPath}/{fileName}";
            string assetDatabaseFile = $"Assets/Resources/{folderPath}/{fileName}.asset";
            string ioFolder = Environment.CurrentDirectory + $"Assets/Resources/{folderPath}";

            //load data using Resources (resourcesFile)
            Settings settings = Resources.Load<Settings>(resourcesFile);

#if UNITY_EDITOR
            //if data does not exists
            if (settings == null)
            {
                //setup data with default values
                settings = CreateInstance<Settings>();

                //create folders using IO (ioFolder)
                if (!Directory.Exists(ioFolder))
                {
                    Directory.CreateDirectory(ioFolder);
                }

                //create file using AssetDatabase (assetDatabaseFile)
                UnityEditor.AssetDatabase.CreateAsset(settings, assetDatabaseFile);
                UnityEditor.AssetDatabase.SaveAssets();
            }
#endif

            return settings;
        }
#if UNITY_EDITOR
        public static UnityEditor.SerializedObject GetSerializedObject()
        {
            return new UnityEditor.SerializedObject(GetScriptableObject());
        }
#endif
    }
}