using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;

namespace AlienRunner
{
    [RequireComponent(typeof(AudioSource))]
    public class SFX : Singleton<SFX>
    {
        [SerializeField]
        private AudioBank audioBank;
        [SerializeField]
        private AudioSource oneShotAudioSource;
        protected override bool InstantiateIfMissing => false;
        protected override bool UseDontDestroyOnLoad => false;

        private SoundEffectPlayer currentMusic;

        private void OnValidate()
        {
            if(oneShotAudioSource == null)
            {
                oneShotAudioSource = GetComponent<AudioSource>();
            }
        }

        public static SoundEffectPlayer PlayStoppable(string name)
        {
            if(!I.audioBank.soundEffects.Any(sfx => sfx.name == name))
            {
                Debug.LogError($"Couldn't play sound effect {name}, since it doesn't exist!");
                return null;
            }
            return new SoundEffectPlayer(I.audioBank.soundEffects.FirstOrDefault(sfx => sfx.name == name));
        }
        public static void Play(string name)
        {
            PlayStoppable(name);
            return;    

            if (!I.audioBank.soundEffects.Any(sfx => sfx.name == name))
            {
                Debug.LogError($"Couldn't play sound effect {name}, since it doesn't exist!");
                return;
            }
            var sfx = I.audioBank.soundEffects.FirstOrDefault(sfx => sfx.name == name);
            if(sfx.clip == null)
            {
                Debug.LogError($"{name} doesn't have an AudioClip.");
                return;
            }
            if(sfx.loop)
            {
                Debug.LogError($"{name} is a looping sound effect. You should use PlayStoppable() to play it.");
                return;
            }
            I.oneShotAudioSource.PlayOneShot(sfx.clip, sfx.volume);
        }
        public static void PlayMusic(string name)
        {
            if (I.currentMusic != null) I.currentMusic.Stop();
            I.currentMusic = PlayStoppable(name);
        }
        public static void PlayOnJoystick(string name, GameObject player)
        {
            if (!I.audioBank.soundEffects.Any(sfx => sfx.name == name))
            {
                Debug.LogError($"Couldn't play sound effect {name}, since it doesn't exist!");
                return;
            }
            var sfx = I.audioBank.soundEffects.FirstOrDefault(sfx => sfx.name == name);
            if (string.IsNullOrEmpty(sfx.url))
            {
                Debug.LogError($"{name} doesn't have a URL.");
                return;
            }
            player.GetComponent<PlayerInput>().Controller.PlaySound(sfx);
        }
    }
    public class SoundEffectPlayer
    {
        private AudioSource source;

        public SoundEffectPlayer(SoundEffect effect)
        {
            GameObject gameObject = new GameObject($"{effect.name}");
            source = gameObject.AddComponent<AudioSource>();
            source.playOnAwake = false;
            source.loop = effect.loop;
            source.volume = effect.volume;
            source.clip = effect.clip;
            if (!effect.dontUseRandomPitch)
            {
                source.pitch = UnityEngine.Random.Range(0.95f, 1.05f);
            }
            source.Play();
            if (!effect.loop)
            {
                GameObject.Destroy(source.gameObject, effect.clip.length + 1f);
            }
        }

        public void Stop()
        {
            if (source == null) 
                return;
            source.Stop();
            GameObject.Destroy(source.gameObject);
        }
    }
}
