using System.Collections;
using System.Collections.Generic;
using UnityEngine;

namespace AlienRunner
{
    public class BackgroundItem : MonoBehaviour
    {
        [SerializeField]
        private float speed = 1;

        private Camera mainCamera;
        private SpriteRenderer spriteRenderer;

        public Vector2 randomSizeRange;
        public Vector2 randomYRange;
        public bool randomFlipX;

        private void Awake()
        {
            mainCamera = Camera.main;
            spriteRenderer = GetComponent<SpriteRenderer>();
            if (spriteRenderer == null)
            {
                spriteRenderer = GetComponentInChildren<SpriteRenderer>();
            }
        }

        private void Update()
        {
            if (GameState.I.CurrentState == GameState.State.Running)
            {
                transform.localPosition += Vector3.left * GameState.I.GameSpeed * speed * Time.deltaTime;
            }

            if (!spriteRenderer.IsVisibleMainCamera() && transform.position.x < mainCamera.ViewportToWorldPoint(Vector3.zero).x)
            {
                float x = mainCamera.ViewportToWorldPoint(Vector3.one).x + spriteRenderer.bounds.size.x;
                transform.position = new Vector3(x, Random.Range(randomYRange.x, randomYRange.y), transform.position.z);
                transform.localScale = Vector3.one * randomSizeRange;

                if (randomFlipX)
                {
                    spriteRenderer.flipX = Random.value > .5f;
                }
            }
        }
    }
}
