using EventGames;
using UnityEngine;

namespace AlienRunner
{
    public class EventGamesController : ControllerBase
    {
        private bool fly;
        private bool usePowerup;

        public EventGamesController(int id) : base(id)
        {
            RuntimeManager.I.onPlayerInput.AddListener(OnPlayerInput);
        }

        private void OnPlayerInput(int id, string message)
        {
            if (Id == id)
            {
                MessageBase messageBase = MessageBase.FromJson(message);

                if (messageBase.command == "fly_up")
                {
                    fly = true;
                }
                else if (messageBase.command == "fly_down")
                {
                    fly = false;
                }
                else if (messageBase.command == "use_power_up")
                {
                    usePowerup = true;
                }
            }
        }
        public override bool Fly()
        {
            return fly;
        }
        public override bool UsePowerUp()
        {
            if (usePowerup)
            {
                usePowerup = false;
                return true;
            }
            return false;
        }
        public override void AddPowerUp(PowerUpBase power)
        {
            RuntimeManager.I.Send(new EventGamesAddPowerupMessage(Id, power).ToJson());
        }
        public override void Waiting()
        {
            RuntimeManager.I.Send(new EventGamesWaitingMessage(Id).ToJson());
        }
        public override void GameIsFull()
        {
            RuntimeManager.I.Send(new EventGamesGameIsFullMessage(Id).ToJson());
        }
        public override void StartGame()
        {
            RuntimeManager.I.Send(new EventGamesStartGameMessage(Id).ToJson());
        }
        public override void GameOver(int placement)
        {
            RuntimeManager.I.Send(new EventGamesGameOverMessage(Id, placement + 1).ToJson());
        }

        public override void PlaySound(SoundEffect sfx)
        {
            RuntimeManager.I.Send(new EventGamesPlaySoundMessage(Id, sfx).ToJson());
        }
    }
}