using System.Collections;
using System.Collections.Generic;
using UnityEngine;

namespace AlienRunner
{
    public class DroppedPickupBehaviour : MonoBehaviour
    {
        [SerializeField]
        private float timeBeforePickupable;
        [SerializeField]
        private float moveDistance;
        [SerializeField]
        private float moveSpeed;
        [SerializeField]
        private float lifetime;

        private float moveDistanceRemaining;
        private float pickupableTimer;

        private Vector3 moveDirection;
        private void Awake()
        {
            moveDistanceRemaining = moveDistance;
            pickupableTimer = timeBeforePickupable;

            float angle = Random.Range(100, 90 + 180 - 10) * Mathf.Deg2Rad;
            moveDirection = new Vector3(Mathf.Cos(angle), Mathf.Sin(angle), 0f);

            GetComponent<PointPickupBehaviour>().CanBePickedUp = false;
        }

        // Update is called once per frame
        void Update()
        {
            if(pickupableTimer > 0f)
            {
                pickupableTimer -= Time.deltaTime;
                if(pickupableTimer <= 0f)
                {
                    GetComponent<PointPickupBehaviour>().CanBePickedUp = true;
                }
            }
            if (moveDistanceRemaining > 0)
            {
                float moveAmount = moveSpeed * Time.deltaTime;
                transform.position += moveDirection * moveAmount;
                moveDistanceRemaining -= moveAmount;
            }
            if(lifetime > 0f)
            {
                lifetime -= Time.deltaTime;
                if(lifetime <= 0f)
                {
                    Destroy(gameObject);
                }
            }
        }
    }
}