using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using Spine.Unity;

namespace AlienRunner
{
    public class EelBehaviour : MonoBehaviour
    {
        [Header("Timers")]
        [SerializeField]
        private float idleTime;
        [SerializeField]
        private float chargeUpTime;
        [SerializeField]
        private float firingTime;
        [SerializeField]
        private float chargeDownTime;

        [Header("Animations")]
        [SerializeField]
        private SkeletonAnimation skeletonAnimation;
        [Space]
        [SerializeField]
        private AnimationReferenceAsset idle;
        [SerializeField]
        private AnimationReferenceAsset chargingUp;
        [SerializeField]
        private AnimationReferenceAsset firing;
        [SerializeField]
        private AnimationReferenceAsset chargingDown;


        private bool laserActive;

        public event Action OnLaserStart;
        public event Action OnLaserEnd;

        private Vector2 startPos;
        private Vector2 currentTargetPos;

        private void Start()
        {
            startPos = transform.position;
            currentTargetPos = startPos;
            StartCoroutine(CycleCoroutine());
        }

        private void Update()
        {
            if (Vector2.Distance(transform.position, currentTargetPos) <= 0.2f)
            {
                float x = UnityEngine.Random.Range(-1f, 1f);
                float y = UnityEngine.Random.Range(-1f, 1f);
                currentTargetPos = new Vector2(startPos.x + x, startPos.y + y);
            }
            if (!laserActive)
                transform.position = Vector2.Lerp(transform.position, currentTargetPos, 1f * Time.deltaTime);

        }

        private IEnumerator CycleCoroutine()
        {
            while (true)
            {
                //Idle
                skeletonAnimation.state.SetAnimation(0, idle, true);
                yield return new WaitForSeconds(idleTime);

                //Charging up
                Spine.TrackEntry track = skeletonAnimation.state.SetAnimation(0, chargingUp, false);
                track.TimeScale = chargingUp.Animation.Duration / chargeUpTime;
                yield return new WaitForSpineAnimationComplete(track);

                //Firing Laser
                skeletonAnimation.state.SetAnimation(0, firing, true);
                OnLaserStart?.Invoke();
                laserActive = true;
                yield return new WaitForSeconds(firingTime);

                //Charging down
                track = skeletonAnimation.state.SetAnimation(0, chargingDown, false);
                track.TimeScale = chargingDown.Animation.Duration / chargeDownTime;
                OnLaserEnd?.Invoke();
                laserActive = false;
                yield return new WaitForSpineAnimationComplete(track);
            }
        }

        private void OnTriggerEnter2D(Collider2D collision)
        {
            if (!laserActive)
                return;
            if (collision.gameObject.tag.ToLower() == "player")
                collision.gameObject.GetComponent<PlayerMovement>().Hit(PlayerMovement.HitType.Stun);

        }


    }
}
