using System.Collections;
using System.Collections.Generic;
using UnityEngine;

namespace AlienRunner
{
    public class EelLaser : MonoBehaviour
    {
        [SerializeField]
        private Transform target;
        [SerializeField]
        private EelBehaviour behaviour;
        [SerializeField]
        private GameObject laserPrefab;

        private GameObject laserObject;
        private LineRenderer laserLineRenderer;

        private List<GameObject> zappedPlayers = new List<GameObject>();

        private void OnValidate()
        {
            if (behaviour == null)
                behaviour = GetComponent<EelBehaviour>();
        }
        // Start is called before the first frame update
        void Start()
        {
            behaviour.OnLaserStart += InstantiateOrEnableLaser;
            behaviour.OnLaserEnd += () =>
            {
                if (laserObject != null)
                {
                    laserObject.SetActive(false);
                    zappedPlayers.Clear();
                }
            };
        }

        private void Update()
        {
            if (laserObject != null && laserObject.activeInHierarchy)
            {
                Vector2 dir = (target.position - transform.position).normalized;
                float dist = Vector2.Distance(transform.position, target.position);
                RaycastHit2D[] hits = Physics2D.RaycastAll(transform.position, dir, dist);
                foreach (var hit in hits)
                {
                    if (hit.transform.tag.ToLower() == "player" && !zappedPlayers.Contains(hit.transform.gameObject))
                    {
                        zappedPlayers.Add(hit.transform.gameObject);
                        hit.transform.GetComponent<PlayerMovement>().Hit(PlayerMovement.HitType.Stun);
                    }
                }
            }
        }

        void InstantiateOrEnableLaser()
        {
            if (laserObject == null)
            {
                laserObject = Instantiate(laserPrefab);
                laserLineRenderer = laserObject.GetComponent<LineRenderer>();
                laserLineRenderer.positionCount = 2;
                laserLineRenderer.startWidth = 2;
                laserLineRenderer.endWidth = 2;
            }
            laserLineRenderer.SetPositions(new Vector3[] { transform.position, target.position });
            zappedPlayers.Clear();
            laserObject.SetActive(true);
            SFX.Play("eel_laser");
        }
    }
}
