using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using Spine.Unity;
using System.Linq;

namespace AlienRunner
{
    public class FartMonsterBehaviour : MonoBehaviour
    {
        [SerializeField]
        private bool debug;
        [Header("Stats")]
        [SerializeField]
        private float goUpTime;
        [SerializeField]
        private float eatingTime;
        [SerializeField]
        private float goDownTime;
        [SerializeField]
        private float fartTime;
        [SerializeField]
        private int closeRank;

        [Space]
        [SerializeField]
        private SkeletonAnimation skeletonAnimationForeground;
        [SerializeField]
        private SkeletonAnimation skeletonAnimationBackground;


        [Header("Animations")]
        [SerializeField]
        private AnimationReferenceAsset downIdle;
        [SerializeField]
        private AnimationReferenceAsset eat;
        [SerializeField]
        private AnimationReferenceAsset fart;
        [SerializeField]
        private AnimationReferenceAsset goDown;
        [SerializeField]
        private AnimationReferenceAsset goUp;
        [SerializeField]
        private AnimationReferenceAsset upIdle;

        [Header("Points")]
        [SerializeField]
        private Transform suckPoint;
        [SerializeField]
        private Transform poopPoint;

        private bool openMouth;

        private int currentlyBeingSuckedInCount = 0;

        public bool HasSeenPlayer { get; set; }

        // Start is called before the first frame update
        void Start()
        {
            StartCoroutine(CycleCoroutine());
        }
        private void Log(string msg)
        {
            if (!debug)
                return;
            Debug.Log(msg);
        }
        
        IEnumerator CycleCoroutine()
        {
            //downIdle
            SetAnimation(0, downIdle, true);

            yield return new WaitUntil(() => HasSeenPlayer);


            //go up
            Log("Going up.");
            SFX.Play("fart_monster_popup");
            yield return new WaitForSpineAnimationComplete(SetAnimation(0, upIdle, false, goUpTime));

            //up idle
            Log("Fart Monster is up.");
            SetAnimation(0, upIdle, true);
            openMouth = true;

            yield return new WaitUntil(() =>
            {
                return suckPoint.position.x <= Players.I.RankToPosition(closeRank);
            });
            yield return new WaitUntil(() => currentlyBeingSuckedInCount < 0);

            //Go down
            openMouth = false;
            Log("Going down to digest");
            yield return new WaitForSpineAnimationComplete(SetAnimation(0, goDown, false, goDownTime));
            SetAnimation(0, downIdle, true);
        }

        IEnumerator DigestPlayer(PlayerState playerState)
        {
            SFX.Play("fart_monster_suck");
            currentlyBeingSuckedInCount++;

            int originalRank = Players.I.GetRank(playerState.gameObject);
            int targetRank = Mathf.Max(originalRank - 2, 0);

            float suckInTime = 0.2f;
            float originalDistance = Vector3.Distance(playerState.transform.position, suckPoint.position);
            float suckInSpeed = originalDistance / suckInTime;

            while (Vector3.Distance(playerState.transform.position, suckPoint.position) > 0.1f)
            {
                playerState.transform.position = Vector3.MoveTowards(playerState.transform.position, suckPoint.position, suckInSpeed * Time.deltaTime);
                yield return null;
            }
            SFX.Play("fart_monster_eat");

            currentlyBeingSuckedInCount--;
            while (poopPoint.position.x >= Players.I.RankToPosition(targetRank))
            {
                playerState.transform.position = new Vector3(Players.I.RankToPosition(targetRank), Camera.main.transform.position.y - (Camera.main.orthographicSize + 2), playerState.transform.position.z);
                yield return null;
            }
            Log("Waiting to be allowed to fart out a player");

            Log("Preparing fart");

            SetAnimation(2, fart, false, fartTime);

            yield return new WaitForSpineEvent(skeletonAnimationForeground, "spawn_player");

            playerState.Set(PlayerState.State.Pooped);
            playerState.transform.position = poopPoint.position;
            SFX.Play("fart_monster_poop");
        }

        private void OnTriggerEnter2D(Collider2D collision)
        {
            if (!openMouth)
            {
                return;
            }
            if (collision.gameObject.tag.ToLower() != "player")
            {
                return;
            }
            PlayerState playerState = collision.gameObject.GetComponent<PlayerState>();
            if (playerState.IsEaten)
            {
                return;
            }
            playerState.Set(PlayerState.State.Eaten);

            playerState.StartCoroutine(DigestPlayer(playerState));
            //SetAnimation(1, eat, false, eatingTime);
        }

        Spine.TrackEntry SetAnimation(int track, AnimationReferenceAsset animation, bool loop, float time = 0)
        {
            float timeScale = time > 0 ? animation.Animation.Duration / time : 1;
            skeletonAnimationBackground.state.SetAnimation(track, animation, loop).TimeScale = timeScale;
            Spine.TrackEntry foreground = skeletonAnimationForeground.state.SetAnimation(track, animation, loop);
            foreground.TimeScale = timeScale;
            return foreground;
        }
    }
}
