using EventGames;
using UnityEngine;

namespace AlienRunner
{
    public class GameSetupHandler : MonoBehaviour
    {
        [SerializeField]
        private bool debug;

        [SerializeField]
        private float timeBeforeAutoStart;

        private float setupTimer;
        public float TimerInSeconds
        {
            get
            {
                return Mathf.RoundToInt(setupTimer);
            }
        }

        private ConnectionBase connectionhandler;

        private void Awake()
        {
            if (debug)
            {
                Debug.Log(GameState.I.Mode);
            }

            switch (GameState.I.Mode)
            {
                case GameState.ControllerMode.LocalManual:
                    connectionhandler = new LocalManualConnection();
                    break;
                case GameState.ControllerMode.LocalAutomatic:
                    connectionhandler = new LocalAutomaticConnection();
                    break;
                case GameState.ControllerMode.EventGames:
                    connectionhandler = new EventGamesConnection();
                    break;
                case GameState.ControllerMode.TestRunner:
                    break;
            }

            if (connectionhandler != null)
            {
                connectionhandler.Awake();
                connectionhandler.OnPlayerConnect += OnPlayerConnect;
                connectionhandler.OnPlayerDisconnect += OnPlayerDisconnect;
            }

            setupTimer = timeBeforeAutoStart;
        }

        private void OnPlayerConnect(int id, int tint, string name)
        {
            //game is full
            if (Players.I.PlayerCount >= Players.max || GameState.I.CurrentState == GameState.State.End)
            {
                GameObject tmpPlayer = new GameObject();
                PlayerInput playerInput = tmpPlayer.AddComponent<PlayerInput>();
                playerInput.Setup(id);
                playerInput.Controller.GameIsFull();
                Destroy(tmpPlayer);
            }
            //waiting for game to start
            else
            {
                GameObject player = Players.I.AddPlayer(id, tint, name);
            }
        }
        private void OnPlayerDisconnect(int id)
        {
            if (GameState.I.CurrentState != GameState.State.End)
            {
                Players.I.RemovePlayer(id);
            }
        }

        private void Start()
        {
            Application.targetFrameRate = 30;
            connectionhandler?.Start();
            SFX.PlayMusic("music");

        }
        private void Update()
        {
            connectionhandler?.Update();

            if (GameState.I.CurrentState == GameState.State.AwaitingPlayers && Players.I.PlayerCount != 0)
            {
                setupTimer -= Time.deltaTime;

                if (setupTimer <= 0)
                {
                    GameState.I.SetGameState(GameState.State.Running);
                }
            }
        }
    }
}
