using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;

namespace AlienRunner
{
    public class GameState : Singleton<GameState>
    {
        [SerializeField]
        private bool debug;

        public enum State { AwaitingPlayers, Running, End }
        public enum ControllerMode
        {
            LocalManual,
            LocalAutomatic,
            EventGames,
            TestRunner
        }

        //public static GameState I { get; private set; }

        [SerializeField]
        private ControllerMode mode;
        public ControllerMode Mode { get => mode; }
        [SerializeField]
        private float gameSpeed;

        public event Action OnGameStart;
        public event Action OnGameEnd;

        public State CurrentState { get; private set; }
        public float GameSpeed { get => gameSpeed; }

        protected override bool InstantiateIfMissing => true;
        protected override bool UseDontDestroyOnLoad => false;

        protected override void Awake()
        {
            CurrentState = State.AwaitingPlayers;
            if (debug)
            {
                Debug.Log($"Controller Mode: {Mode}");
            }
            base.Awake();
        }

        private void Update()
        {
            if (debug)
            {
                Debug.Log($"Game state: {CurrentState}");
            }
        }

        public void SetGameState(State state)
        {
            if (CurrentState != state)
            {
                CurrentState = state;
                switch (state)
                {
                    case State.Running:
                        OnGameStart?.Invoke();
                        break;
                    case State.End:
                        OnGameEnd?.Invoke();
                        break;  
                }
            }
        }
    }
}
