using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.SceneManagement;

namespace AlienRunner
{
    public class GameTimers : MonoBehaviour
    {
        [Header("Timers")]
        [SerializeField]
        private float gameDuration;
        [SerializeField]
        private float timeBeforeRestart;
        [SerializeField]
        private bool restartGameAutomatically;

        [SerializeField]
        private GameObject GameEndUI;
        [SerializeField]
        private TimerDisplay GameTimerDisplay;
        [SerializeField]
        private TimerDisplay RestartTimerDisplay;


        void Start()
        {
            GameEndUI.SetActive(false);

            GameState.I.OnGameStart += () =>
            {
                GameTimerDisplay.Activate(gameDuration);
            };
            GameState.I.OnGameEnd += () =>
            {
                GameEndUI.SetActive(true);
                if (restartGameAutomatically)
                {
                    RestartTimerDisplay.Activate(timeBeforeRestart);
                }
            };
        }

        void Update()
        {
            if (GameState.I.CurrentState == GameState.State.Running)
            {
                if (GameTimerDisplay.TimeIsUp)
                {
                    SFX.PlayMusic("game_over_fanfare");
                    GameTimerDisplay.Deactivate();
                    GameState.I.SetGameState(GameState.State.End);
                }
            }
            if (GameState.I.CurrentState == GameState.State.End)
            {
                if (restartGameAutomatically && RestartTimerDisplay.TimeIsUp)
                {
                    SceneManager.LoadScene("Game");
                }
            }
        }
    }
}
