using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;

namespace AlienRunner
{
    public class LevelGenerator : MonoBehaviour
    {
        [SerializeField]
        [Min(-1)]
        private int seed;
        [SerializeField]
        private GameObject[] chunkPrefabs;

        private Camera mainCamera;
        private System.Random random;
        private List<Chunk> activeChunks;

        protected void Awake()
        {
            mainCamera = Camera.main;
            transform.parent = mainCamera.transform;
            transform.localPosition = Vector3.zero;

            activeChunks = new List<Chunk>();
            if (seed == -1)
            {
                random = new System.Random();
            }
            else
            {
                random = new System.Random(seed);
            }
        }

        public void Update()
        {
            //if the first chunk
            if (activeChunks.Count == 0)
            {
                Chunk chunk = GetRandomChunk();
                chunk.transform.position = Vector3.zero;
                activeChunks.Add(chunk);
            }

            while (true)
            {
                //add chunk in front
                Chunk last = activeChunks.Last();
                if (last.transform.position.x <= mainCamera.ViewportToWorldPoint(new Vector3(1, 0, 0)).x)
                {
                    Chunk chunk = GetRandomChunk();
                    chunk.transform.position = last.transform.position + Vector3.right * last.size;
                    activeChunks.Add(chunk);
                    continue;
                }

                //remove chunk at the back
                Chunk first = activeChunks.First();
                if (first.transform.position.x <= mainCamera.ViewportToWorldPoint(new Vector3(0, 0, 0)).x - first.size)
                {
                    Destroy(first.gameObject);
                    activeChunks.RemoveAt(0);
                    continue;
                }

                break;
            }
        }

        private Chunk GetRandomChunk()
        {
            return Instantiate(chunkPrefabs[random.Next(0, chunkPrefabs.Length)], Vector3.zero, Quaternion.identity).GetComponent<Chunk>();
        }

        public Chunk[] GetGetActiveChunks()
        {
            return activeChunks.ToArray();
        }
    }
}
