using UnityEngine;

namespace AlienRunner
{
    public class PlayerInput : MonoBehaviour, IPlayerSetupable
    {
        public ControllerBase Controller { get; private set; }

        private PlayerState playerState;
        
        public void Setup(int id)
        {
            switch (GameState.I.Mode)
            {
                case GameState.ControllerMode.LocalManual:
                    Controller = new LocalManualController(id);
                    break;
                case GameState.ControllerMode.LocalAutomatic:
                    Controller = new LocalAutomaticController(id);
                    break;
                case GameState.ControllerMode.EventGames:
                    Controller = new EventGamesController(id);
                    break;
                case GameState.ControllerMode.TestRunner:
                    Controller = new TestRunnerController(id);
                    break;
            }
            playerState = GetComponent<PlayerState>();
            playerState.onExit += OnStateExit;
            playerState.onEnter += OnStateEnter;
        }

        private void OnStateEnter()
        {
            if (playerState.IsWaiting)
            {
                Controller.Waiting();
            }
            if (playerState.IsGameEnd)
            {
                Controller.GameOver(GetComponent<PlayerScore>().Placement);
            }
        }

        private void OnStateExit()
        {
            if(playerState.IsWaiting)
            {
                Controller.StartGame();
            }
        }

        private void Update()
        {
            Controller?.Update();
        }
        private void OnDestroy()
        {
            playerState.onExit -= OnStateExit;
            playerState.onEnter -= OnStateEnter;
        }
    }
}
