using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;
using UnityEngine.Serialization;

namespace AlienRunner
{
    public class PlayerPowerUp : MonoBehaviour, IPlayerSetupable
    {
        private PlayerInput input;

        private GameObject activePowerUp;
        private bool powerUpIsActive;

        public PowerUpChance[] lastPlaceItems;
        public PowerUpChance[] lowerHalfItems;
        [FormerlySerializedAs("upperHalfItms")]
        public PowerUpChance[] upperHalfItems;
        public PowerUpChance[] firstPlaceItems;

        public void Setup(int id)
        {
            input = GetComponent<PlayerInput>();
        }

        private void Update()
        {
            if (input.Controller.UsePowerUp())
            {
                StartCoroutine(Use());
            }
        }

        public bool AddRandomPowerup()
        {
            if (!powerUpIsActive && activePowerUp == null)
            {
                activePowerUp = GetRandomPowerUpPrefab();
                activePowerUp.GetComponent<PowerUpBase>().Pickup(gameObject);
                input.Controller.AddPowerUp(activePowerUp.GetComponent<PowerUpBase>());
                return true;
            }
            return false;
        }
        private IEnumerator Use()
        {
            if (!powerUpIsActive && activePowerUp != null)
            {
                GameObject go = Instantiate(activePowerUp, transform.position, Quaternion.identity, transform);
                powerUpIsActive = true;
                activePowerUp = null;
                yield return go.GetComponent<PowerUpBase>().Use(gameObject);
                powerUpIsActive = false;

                Destroy(go);
            }
        }
        private PowerUpChance[] GetPowerupChancesArray()
        {
            int totalPlayers = Players.I.PlayerCount;
            int playerRank = Players.I.GetRank(gameObject);
            if (playerRank == 0)
            {
                return lastPlaceItems;
            }
            if(playerRank ==  totalPlayers - 1)
            {
                return firstPlaceItems;
            }
            if(playerRank >= totalPlayers * 0.5f)
            {
                return upperHalfItems;
            }
            return lowerHalfItems;
        }
        private GameObject GetRandomPowerUpPrefab()
        {
            PowerUpChance[] powerUpChances = GetPowerupChancesArray();
            float sum = powerUpChances.Sum(chance => chance.weight);
            float number = Random.Range(0, sum);
            for(int i=0;i<powerUpChances.Length;i++)
            {
                if(number <= powerUpChances[i].weight)
                {
                    return powerUpChances[i].powerUp.gameObject;   
                }
                number -= powerUpChances[i].weight;
            }
            return powerUpChances.FirstOrDefault().powerUp.gameObject;
        }

    }
}
