using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;
using TMPro;

namespace AlienRunner
{
    public class PlayerScore : MonoBehaviour, IPlayerSetupable
    {
        private new PlayerAnimation animation;
        [AllowedStatic]
        private static Action<List<PlayerScore>> updateAllCrowns;

        [SerializeField]
        public int Score { get; private set; }
        [SerializeField]
        private PlayerScoreUI scoreTextElement;

        public int Placement { get; private set; }
        public float scoreMultiplier;
        public float baseScore = 1;

        // Start is called before the first frame update
        public void Setup(int id)
        {
            animation = GetComponent<PlayerAnimation>();
            Score = 0;
            updateAllCrowns += UpdateMyCrown;
        }

        public static void UpdateAllCrowns()
        {
            updateAllCrowns?.Invoke(Players.I.OrderedScores);
        }
        public void AddToScore(int score)
        {
            float multiplier = scoreMultiplier == 0 ? baseScore : baseScore * scoreMultiplier;
            Score += (int)(score * multiplier);

            UpdateAllCrowns();
        }
        public void LoseScore(int score)
        {
            Score -= score;
            if (Score < 0) Score = 0;
            UpdateAllCrowns();
        }

        private void UpdateMyCrown(List<PlayerScore> list)
        {
            Placement = list.IndexOf(this);
            animation.Crown(Placement, Score);
        }

        private void OnDestroy()
        {
            updateAllCrowns -= UpdateMyCrown;
        }
    }
}
