using System.Collections;
using System.Collections.Generic;
using UnityEngine;

namespace AlienRunner
{
    public class PlayerSound : MonoBehaviour
    {
        [SerializeField]
        private PlayerState playerState;

        private SoundEffectPlayer jetpackSound;

        private void OnValidate()
        {
            if (playerState == null)
            {
                playerState = GetComponent<PlayerState>();
            }
        }

        // Start is called before the first frame update
        void Start()
        {
            playerState.onEnter += PlayerState_onEnter;
            playerState.onExit += PlayerState_onExit;

            SFX.Play("player_join");
        }

        private void PlayerState_onExit()
        {
            if (playerState.IsFlying || playerState.IsSpeedBoosting)
            {
                jetpackSound?.Stop();
            }
        }

        private void PlayerState_onEnter()
        {
            if(playerState.IsFlying || playerState.IsSpeedBoosting)
            {
                jetpackSound?.Stop();
                jetpackSound = SFX.PlayStoppable("jetpack");
            }
            if(playerState.IsStunned)
            {
                SFX.Play("player_electrified");
            }
        }

        private void OnDestroy()
        {
            playerState.onEnter -= PlayerState_onEnter;
            playerState.onExit -= PlayerState_onExit;
        }
    }
}
