using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using Spine.Unity;
using System;
using EventGames;

namespace AlienRunner
{
    public class HomingMissile : MonoBehaviour
    {
        [SerializeField]
        private SkeletonAnimation skeletonAnimation;
        [SerializeField]
        private MeshRenderer meshRenderer;
        [SerializeField]
        private GameObject explosionPrefab;
        [SerializeField]
        private float speed;
        [SerializeField]
        private float waitOuside;
        [SerializeField]
        private float startRotation;

        [SerializeField]
        private GameObject playerWhoFired;

        private enum State
        {
            FlyOutsideScreen,
            WaitOutside,
            FlyToPlayer
        }

        internal void Setup(GameObject by)
        {
            this.playerWhoFired = by;

            Color color = Players.I.GetColor(by);
            foreach (var slot in skeletonAnimation.skeleton.Slots)
            {
                if (slot.ToString().Contains("tint_1_body") || slot.ToString().Contains("tint_1_wings"))
                    slot.SetColor(color);
            }
        }

        private State state;

        private void Awake()
        {
            transform.eulerAngles = new Vector3(0, 0, startRotation);
            SFX.Play("homing_mssile_fire");
        }

        private void Update()
        {
            GameObject target = null;
            foreach (var player in Players.I.GameObjects)
            {
                if (target == null || Players.I.GetRank(player) > Players.I.GetRank(target))
                {
                    target = player;
                }
            }

            if (state == State.FlyOutsideScreen)
            {
                transform.position += transform.up * speed * Time.deltaTime;

                if (!meshRenderer.isVisible && transform.position.y > Camera.main.ViewportToWorldPoint(Vector3.up).y)
                {
                    state = State.WaitOutside;
                }
            }
            if (state == State.WaitOutside)
            {
                waitOuside -= Time.deltaTime;
                if (waitOuside < 0)
                {
                    state = State.FlyToPlayer;
                }
            }
            if (state == State.FlyToPlayer)
            {
                Vector3 relative = transform.InverseTransformPoint(target.transform.position);
                float angle = Mathf.Atan2(relative.x, relative.y) * Mathf.Rad2Deg;
                transform.Rotate(0, 0, -angle);

                transform.position += transform.up * speed * Time.deltaTime;
                transform.position = new Vector3(transform.position.x, transform.position.y, target.transform.position.z);

                if (Vector3.Distance(target.transform.position, transform.position) < .3f)
                {
                    SFX.Play("homing_missile_impact");
                    if(target == playerWhoFired)
                    {
                        SFX.Play("homing_missile_self");
                    }
                    target.GetComponent<PlayerMovement>().Hit(PlayerMovement.HitType.Stun);
                    Instantiate(explosionPrefab, transform.position, Quaternion.identity);
                    Destroy(gameObject);
                }
            }
        }
    }
}
