using Spine.Unity;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;

namespace AlienRunner
{
    public class PowerUpBox : MonoBehaviour
    {
        [SerializeField]
        private SkeletonAnimation skeletonAnimation;

        [SerializeField]
        private AnimationReferenceAsset remove;
        [SerializeField]
        private AnimationReferenceAsset idle;

        private bool canBePickedUp;

        [SerializeField]
        private float respawnInterval = 3f;


        private void Awake()
        {
            canBePickedUp = true;
        }

        public void OnTriggerEnter2D(Collider2D other)
        {
            if (!canBePickedUp)
                return;
            if (other.tag.ToLower() == "player")
            {
                SFX.Play("powerup_pickup");
                PlayerPowerUp playerPowerUp = other.GetComponent<PlayerPowerUp>();
                if (playerPowerUp.AddRandomPowerup())
                {
                    StartCoroutine(RemoveCoroutine());
                }
            }
        }

        public IEnumerator RemoveCoroutine()
        {
            canBePickedUp = false;
            skeletonAnimation.state.SetAnimation(0, remove, false).TimeScale = 1f;
            yield return new WaitForSeconds(respawnInterval);
            skeletonAnimation.state.SetAnimation(0, idle, true).TimeScale = 1f;
            canBePickedUp = true;
        }
    }
}
