using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using TMPro;
using System;

namespace AlienRunner
{
    public class TimerDisplay : MonoBehaviour
    {
        [Header("Timer Info")]
        [SerializeField]
        private bool StartActive;
        [Header("Text")]
        [SerializeField]
        private TextMeshProUGUI text;
        [SerializeField]
        private string inactiveText;
        [SerializeField]
        private string activeText;
        [Header("Display Info")]
        [SerializeField]
        private float displayAtPercentage;
        [SerializeField]
        private Color timerGettingCloseColor;
        [SerializeField]
        private Color timeAlmostUpColor;

        private Color baseColor;

        private float duration;
        private float timer;
        private bool active;

        public bool TimeIsUp => timer <= 0;

        private void Start()
        {
            active = StartActive;
            baseColor = text.color;
        }

        private void Update()
        {
            if (active)
            {
                timer -= Time.deltaTime;
                float showText = duration * displayAtPercentage;
                if (timer <= showText)
                {
                    int time = Mathf.CeilToInt(timer);
                    string color = $"#{ColorUtility.ToHtmlStringRGB(baseColor)}";
                    if (time <= showText / 3)
                    {
                        if (timeAlmostUpColor.a != 0)
                            color = $"#{ColorUtility.ToHtmlStringRGB(timeAlmostUpColor)}";
                    }
                    else if (time <= (showText / 3) * 2)
                    {
                        if (timerGettingCloseColor.a != 0)
                        color = $"#{ColorUtility.ToHtmlStringRGB(timerGettingCloseColor)}";
                    }
                    if (string.IsNullOrEmpty(activeText))
                        text.text = $"{timer}";
                    else
                    {
                        string s = activeText.Replace("@time", $"<color={color}>{time}</color>");
                        text.text = s;
                    }
                }
            }
            else
            {
                text.text = inactiveText;
            }
        }

        public void Activate(float duration)
        {
            this.duration = duration;
            timer = duration;
            active = true;
        }

        public void Deactivate()
        {
            active = false;
            Destroy(text.gameObject);
        }

    }
}
