using System.Collections;
using System.Collections.Generic;
using TMPro;
using UnityEngine;
using UnityEngine.UI;

namespace AlienRunner
{
    public class EndScreen : MonoBehaviour
    {
        [SerializeField]
        TextMeshProUGUI winnerText;
        [SerializeField]
        TextMeshProUGUI scoreboardNames;
        [SerializeField]
        TextMeshProUGUI scoreboardScores;

        private List<PlayerScore> players;

        private void OnEnable()
        {
            if (GameState.I.CurrentState == GameState.State.End)
            {
                winnerText.text = "";
                scoreboardNames.text = "";
                scoreboardScores.text = "";
                players = Players.I.OrderedScores;

                //Winner
                SetupWinnerText();

                for (int i = 1; i < players.Count; i++)
                {
                    GameObject player = players[i].gameObject;
                    string name = Players.I.GetColorName(player);
                    int score = players[i].Score;
                    scoreboardNames.text += $"{i + 1}. {name}\n";
                    scoreboardScores.text += $"{score}\n";
                }
            }
        }

        private void SetupWinnerText()
        {
            GameObject winner = players[0].gameObject;
            Color color = Players.I.GetColor(winner);
            string winnerName = Players.I.GetColorName(winner);
            int score = players[0].Score;
            winnerText.text = $"{winnerName} {score}";
            winnerText.color = color;
            Image[] crowns = winnerText.GetComponentsInChildren<Image>();
            foreach (var c in crowns)
            {
                c.color = color;
            }
        }
    }
}
