using System.Collections;
using System.Collections.Generic;
using TMPro;
using UnityEngine;

namespace AlienRunner
{
    public class PlayerScoreUI : MonoBehaviour
    {
        [SerializeField]
        private float growAmount;
        [SerializeField]
        private float growSpeed;
        [SerializeField]
        private float fadeSpeed;
        [SerializeField]
        private float waitBeforeFade;

        private TextMeshPro textElement;
        private PlayerScore playerScore;

        private int targetScore => playerScore.Score;
        private int displayScore;

        private float defaultFontSize;
        private float growSize => defaultFontSize + growAmount;


        // Start is called before the first frame update
        IEnumerator Start()
        {
            playerScore = GetComponentInParent<PlayerScore>();
            textElement = GetComponentInParent<TextMeshPro>();
            textElement.text = "";
            defaultFontSize = textElement.fontSize;

            while (true)
            {
                yield return new WaitUntil(() => targetScore != displayScore);

                int sign = System.Math.Sign(targetScore - displayScore);
                displayScore += sign;

                //Reset everything
                textElement.text = displayScore.ToString();
                textElement.fontSize = defaultFontSize;
                textElement.color = new Color(textElement.color.r, textElement.color.g, textElement.color.b, 1);

                while (growSize != textElement.fontSize)
                {
                    textElement.fontSize = Mathf.MoveTowards(textElement.fontSize, growSize, growSpeed * Time.deltaTime);
                    yield return null;
                }

                if (displayScore != targetScore)
                    continue;
                float timer = 0.5f;
                while (timer > 0)
                {
                    timer -= Time.deltaTime;
                    if (displayScore != targetScore)
                        continue;
                    yield return null;
                }

                Color color = textElement.color;
                float a = 1;
                while (a > 0)
                {
                    a = Mathf.MoveTowards(a, 0, fadeSpeed * Time.deltaTime);
                    textElement.color = new Color(color.r, color.g, color.b, a);
                    yield return null;
                }
                textElement.text = "";
            }
        }


    }
}
