Shader "AlienRunner/PostProcessing/CurvedWorldShader"
{
	Properties
	{
		_MainTex("Texture", 2D) = "white"{}
	}
	SubShader
	{
		Cull Off
		ZWrite Off
		ZTest Always

		Pass
		{
			CGPROGRAM
			#pragma vertex vertex
			#pragma fragment fragment

			#include "UnityCG.cginc"


			sampler2D _MainTex;

			struct vertexInput
			{
				float4 vertex : POSITION;
				float2 uv : TEXCOORD0;
			};

			struct vertexToFragment
			{
				float4 vertex : SV_POSITION;
				float2 uv : TEXCOORD0;
			};

			vertexToFragment vertex(vertexInput input)
			{
				vertexToFragment output;
				output.vertex = UnityObjectToClipPos(input.vertex);
				output.uv = input.uv;
				return output;
			}


			fixed4 fragment(vertexToFragment input) : SV_Target
			{
				float diff = input.uv.x - 0.5;
				float diffSign = sign(diff);
				float diffAbs = abs(diff);
				float diffNormalized = smoothstep(-0.2, 1.2, diffAbs);
				float diffSqr = diffNormalized * diffNormalized;
				float diffScaled = diffSqr * 0.1;

				float unScaleFactor = 1 - input.uv.y;
				diffScaled *= unScaleFactor;

				//return fixed4(diffScaled, diffScaled, diffScaled, diffScaled);
				
				input.uv.y += diffScaled;

				fixed4 color = tex2D(_MainTex, input.uv);
				return color;
			}
			ENDCG
		}
	}
}
