using System.Collections;
using NUnit.Framework;
using UnityEngine;
using UnityEngine.TestTools;

namespace AlienRunner.Tests
{
    public class CoinPickupTests
    {
        [UnityTest]
        public IEnumerator CoinPickup_ScoreIsAdded()
        {
            //-Arrange
            yield return Arrange.Reset();
            Arrange.SpawnPrefab("Test Prefabs/Scene");
            //player
            Players.I.AddPlayer(0, 0, "");
            GameObject player = Players.I.GameObjects[0];
            PlayerScore score = player.GetComponent<PlayerScore>();
            //Spawn one coin in front of the player
            GameObject coin = Arrange.SpawnPrefab("Environment/Coin");
            coin.transform.position = player.transform.position;
            int pointIncreasePerCoin = coin.GetComponent<PointPickupBehaviour>().ScoreIncrease;

            //-Act
            GameState.I.SetGameState(GameState.State.Running);
            yield return null;

            //-Assert
            int expectedScore = pointIncreasePerCoin;
            Assert.AreEqual(expectedScore, score.Score);
        }

        [UnityTest]
        public IEnumerator CoinPickup_ScoreIsAdded_WithMultiplier()
        {
            //-Arrange
            yield return Arrange.Reset();
            Arrange.SpawnPrefab("Test Prefabs/Scene");
            //player
            Players.I.AddPlayer(0, 0, "");
            GameObject player = Players.I.GameObjects[0];
            PlayerScore score = player.GetComponent<PlayerScore>();
            //Spawn one coin in front of the player
            GameObject coin = Arrange.SpawnPrefab("Environment/Coin");
            coin.transform.position = player.transform.position;

            int pointIncreasePerCoin = coin.GetComponent<PointPickupBehaviour>().ScoreIncrease;

            //-Act
            GameState.I.SetGameState(GameState.State.Running);
            float multiplier = 2;
            score.scoreMultiplier += multiplier;
            yield return null;

            //-Assert
            Assert.AreEqual(multiplier, score.Score);
        }
        [UnityTest]
        public IEnumerator CoinPickup_HighestScoreIsLeader()
        {
            //-Arrange
            yield return Arrange.Reset();
            Arrange.SpawnPrefab("Test Prefabs/Scene");
            //players
            Players.I.AddPlayer(0, 0, "");
            GameObject player1 = Players.I.GameObjects[0];
            PlayerScore score1 = player1.GetComponent<PlayerScore>();
            Players.I.AddPlayer(1, 0, "");
            GameObject player2 = Players.I.GameObjects[1];
            PlayerScore score2 = player1.GetComponent<PlayerScore>();
            player2.transform.position = new Vector3(player1.transform.position.x - 2, player1.transform.position.y, 0);
            //coin
            GameObject coin = Arrange.SpawnPrefab("Environment/Coin");
            coin.transform.position = player1.transform.position;

            //-Act
            GameState.I.SetGameState(GameState.State.Running);
            yield return null;

            //-Assert
            bool expected = true;
            bool actual = score1.Placement == 0;
            Assert.AreEqual(expected, actual);
        }
        [UnityTest]
        public IEnumerator CoinPickup_LowerScoreIsNotLeader()
        {
            //-Arrange
            yield return Arrange.Reset();
            Arrange.SpawnPrefab("Test Prefabs/Scene");
            //players
            Players.I.AddPlayer(0, 0, "");
            GameObject player1 = Players.I.GameObjects[0];
            PlayerScore score1 = player1.GetComponent<PlayerScore>();
            Players.I.AddPlayer(1, 0, "");
            GameObject player2 = Players.I.GameObjects[1];
            PlayerScore score2 = player2.GetComponent<PlayerScore>();
            //Spawn a coin in front of the players
            GameObject coin = Arrange.SpawnPrefab("Environment/Coin");
            coin.transform.position = player2.transform.position;

            //-Act
            GameState.I.SetGameState(GameState.State.Running);
            yield return null;

            //-Assert
            bool expected = true;
            bool actual = score2.Placement == 0;
            Assert.AreEqual(expected, actual);
        }
    }
}
