using System.Collections;
using System.Collections.Generic;
using NUnit.Framework;
using UnityEngine;
using UnityEngine.TestTools;

namespace AlienRunner.Tests
{
    public class DeathTest
    {
        [UnityTest]
        public IEnumerator PlayerStateIsDead()
        {
            //-Arrange
            yield return Arrange.Reset();
            Arrange.SpawnPrefab("Test Prefabs/Scene");
            Players.I.AddPlayer(0, 0, "");
            GameObject player = Players.I.GameObjects[0];
            PlayerState state = player.GetComponent<PlayerState>();

            //-Act
            GameState.I.SetGameState(GameState.State.Running);
            player.transform.position = new Vector3(Camera.main.ViewportToWorldPoint(Vector3.zero).x - 5, 0, 0);
            yield return null;

            //-Assert
            Assert.IsTrue(state.IsDead);
        }

        [UnityTest]
        public IEnumerator PlayerIsInvincible()
        {
            //-Arrange
            yield return Arrange.Reset();
            Arrange.SpawnPrefab("Test Prefabs/Scene");
            Players.I.AddPlayer(0, 0, "");
            GameObject player = Players.I.GameObjects[0];
            PlayerState state = player.GetComponent<PlayerState>();

            //-Act
            GameState.I.SetGameState(GameState.State.Running);
            state.Set(PlayerState.State.Dead);
            yield return new WaitForSeconds(state.RespawnTime + 0.1f);

            //-Assert
            Assert.AreEqual(true, state.IsInvincible);
        }

        [UnityTest]
        public IEnumerator PlayerStopsBeingInvincible()
        {
            //-Arrange
            yield return Arrange.Reset();
            Arrange.SpawnPrefab("Test Prefabs/Scene");
            Players.I.AddPlayer(0, 0, "");
            GameObject player = Players.I.GameObjects[0];
            PlayerState state = player.GetComponent<PlayerState>();

            //-Act
            GameState.I.SetGameState(GameState.State.Running);
            state.Set(PlayerState.State.Dead);
            yield return new WaitForSeconds(state.RespawnTime + state.InvincibleTime + 1f);

            //-Assert
            Assert.AreEqual(false, state.IsInvincible);
        }

        [UnityTest]
        public IEnumerator PlayerRespawns()
        {
            //-Arrange
            yield return Arrange.Reset();
            Arrange.SpawnPrefab("Test Prefabs/Scene");
            Players.I.AddPlayer(0, 0, "");
            GameObject player = Players.I.GameObjects[0];
            PlayerState state = player.GetComponent<PlayerState>();

            //-Act
            GameState.I.SetGameState(GameState.State.Running);
            state.Set(PlayerState.State.Dead);
            yield return null;
            yield return new WaitForSeconds(state.RespawnTime);

            //-Assert
            Assert.IsTrue(state.IsMoving);
            Assert.Less(Camera.main.ViewportToWorldPoint(Vector3.zero).x, player.transform.position.x);
        }
        
        [UnityTest]
        public IEnumerator PlayerRespawnsInLastPosition()
        {
            //-Arrange
            yield return Arrange.Reset();
            Arrange.SpawnPrefab("Test Prefabs/Scene");
            Players.I.AddPlayer(0, 0, "");
            GameObject player1 = Players.I.GameObjects[0];
            PlayerState state = player1.GetComponent<PlayerState>();
            Players.I.AddPlayer(0, 0, "");
            GameObject player2 = Players.I.GameObjects[1];
            player2.transform.position = new Vector2(player1.transform.position.x - 2, 0);
            yield return null;

            //-Act
            GameState.I.SetGameState(GameState.State.Running);
            state.Set(PlayerState.State.Dead);
            yield return new WaitForSeconds(state.RespawnTime -0.1f);

            //-Assert
            Assert.AreEqual(0, Players.I.GetRank(player1));
        }
    }
}
