using System.Collections;
using System.Collections.Generic;
using NUnit.Framework;
using UnityEngine;
using UnityEngine.TestTools;

namespace AlienRunner.Tests
{
    public class GameStateTests
    {
        // A UnityTest behaves like a coroutine in Play Mode. In Edit Mode you can use
        // `yield return null;` to skip a frame.
        [UnityTest]
        public IEnumerator OnGameStart_GameStateIsRunning()
        {
            //-Arrange
            yield return Arrange.Reset();
            Arrange.SpawnPrefab("Test Prefabs/Scene");

            //-Act
            GameState.I.SetGameState(GameState.State.Running);

            //-Assert
            Assert.AreEqual(GameState.I.CurrentState, GameState.State.Running);
        }
        [UnityTest]
        public IEnumerator OnGameEnd_GameStateIsEnd()
        {
            //-Arrange
            yield return Arrange.Reset();
            Arrange.SpawnPrefab("Test Prefabs/Scene");

            //-Act
            yield return null;
            GameState.I.SetGameState(GameState.State.End);

            //-Assert
            Assert.AreEqual(GameState.I.CurrentState, GameState.State.End);
        }

        [UnityTest]
        public IEnumerator OnGameStart_PlayerGravityScaleIsNotZero()
        {
            //-Arrange
            yield return Arrange.Reset();
            Arrange.SpawnPrefab("Test Prefabs/Scene");

            //Makes sure that FixedUpdate is called before we spawn a player to avoid null-reference exceptions in PlayerController.FixedUpdate
            yield return new WaitForFixedUpdate();

            Players.I.AddPlayer(0, 0, "");
            GameObject player = Players.I.GameObjects[0];

            //For some reason gravity doesn't work properly if we don't do this? No idea why...
            yield return null;

            //-Act
            GameState.I.SetGameState(GameState.State.Running);
            yield return new WaitForSeconds(1);

            //-Assert
            Assert.AreNotEqual(0f, player.GetComponent<Rigidbody2D>().gravityScale);
        }


    }
}
