using System.Collections;
using System.Collections.Generic;
using System.Linq;
using NUnit.Framework;
using UnityEngine;
using UnityEngine.TestTools;

namespace AlienRunner.Tests
{
    public class LevelGeneratorTest
    {
        [UnityTest]
        public IEnumerator FirstChunkIsAtZero()
        {
            //-Arrange
            yield return Arrange.Reset();
            Arrange.SpawnPrefab("Main Camera");
            LevelGenerator levelGenerator = Arrange.SpawnPrefab("Level/_LevelGenerator").GetComponent<LevelGenerator>();

            //-Act
            yield return null;

            //-Assert
            Chunk firstChunk = levelGenerator.GetGetActiveChunks()[0];
            Assert.AreEqual(firstChunk.transform.position, Vector3.zero);
        }

        [UnityTest]
        public IEnumerator SecondChunkPosition()
        {
            //-Arrange
            yield return Arrange.Reset();
            Arrange.SpawnPrefab("Main Camera");
            LevelGenerator levelGenerator = Arrange.SpawnPrefab("Level/_LevelGenerator").GetComponent<LevelGenerator>();

            //-Act
            yield return null;

            //-Assert
            Chunk firstChunk = levelGenerator.GetGetActiveChunks()[0];
            Chunk secondChunk = levelGenerator.GetGetActiveChunks()[1];
            Assert.AreEqual(secondChunk.transform.position, Vector3.right * firstChunk.size);
        }

        [UnityTest]
        public IEnumerator NewChunksWhenMoving()
        {
            //-Arrange
            yield return Arrange.Reset();
            Arrange.SpawnPrefab("Main Camera");
            LevelGenerator levelGenerator = Arrange.SpawnPrefab("Level/_LevelGenerator").GetComponent<LevelGenerator>();

            //-Act
            yield return null;
            Chunk chunk1 = levelGenerator.GetGetActiveChunks()[0];
            levelGenerator.transform.position += Vector3.right * chunk1.size;
            yield return null;

            //-Assert
            Chunk chunk2 = levelGenerator.GetGetActiveChunks()[0];
            Chunk chunk3 = levelGenerator.GetGetActiveChunks()[1];
            Assert.AreEqual(chunk3.transform.position, chunk2.transform.position + Vector3.right * chunk2.size);
        }

        [UnityTest]
        public IEnumerator ThereAreAlwaysSomeChunks()
        {
            //-Arrange
            yield return Arrange.Reset();
            GameObject camera = Arrange.SpawnPrefab("Main Camera");
            LevelGenerator levelGenerator = Arrange.SpawnPrefab("Level/_LevelGenerator").GetComponent<LevelGenerator>();

            //-Act
            camera.transform.position += Vector3.right * 10;
            yield return null;

            //-Assert
            Assert.Greater(levelGenerator.GetGetActiveChunks().Length, 0);
        }

        [UnityTest]
        public IEnumerator NewChunksAreSpawnedAfterMoving()
        {
            //-Arrange
            yield return Arrange.Reset();
            GameObject camera = Arrange.SpawnPrefab("Main Camera");
            LevelGenerator levelGenerator = Arrange.SpawnPrefab("Level/_LevelGenerator").GetComponent<LevelGenerator>();

            //-Act
            camera.transform.position += Vector3.right * 10;
            Chunk[] chunksMove1 = levelGenerator.GetGetActiveChunks();
            yield return null;

            camera.transform.position += Vector3.right * 10;
            Chunk[] chunksMove2 = levelGenerator.GetGetActiveChunks();
            yield return null;

            //-Assert
            foreach (var chunk in chunksMove1)
            {
                Assert.IsFalse(chunksMove2.Contains(chunk));
            }
        }

        [UnityTest]
        public IEnumerator ChunksAreRemoved()
        {
            //-Arrange
            yield return Arrange.Reset();
            GameObject camera = Arrange.SpawnPrefab("Main Camera");
            LevelGenerator levelGenerator = Arrange.SpawnPrefab("Level/_LevelGenerator").GetComponent<LevelGenerator>();

            //-Act
            camera.transform.position += Vector3.right * 55;
            yield return null;

            //-Assert
            foreach (var chunk in levelGenerator.GetGetActiveChunks())
            {
                Assert.IsFalse(chunk.transform.position.x < Camera.main.ViewportToWorldPoint(new Vector3(0, 0, 0)).x - chunk.size);
            }
        }
    }
}
