using System.Collections;
using System.Collections.Generic;
using NUnit.Framework;
using UnityEngine;
using UnityEngine.TestTools;

namespace AlienRunner.Tests
{
    public class RankTests
    {
        [UnityTest]
        public IEnumerator SpawnIdealPosition()
        {
            //-Arrange
            yield return Arrange.Reset();
            Arrange.SpawnPrefab("Test Prefabs/Scene");
            Arrange.SpawnPrefab("Test Prefabs/LevelGeneratorCoinsOnly");
            Players.I.AddPlayer(0, 0, "");
            Players.I.AddPlayer(1, 0, "");
            Players.I.AddPlayer(2, 0, "");
            Players.I.AddPlayer(3, 0, "");
            GameObject player1 = Players.I.GameObjects[0];
            GameObject player2 = Players.I.GameObjects[1];
            GameObject player3 = Players.I.GameObjects[2];
            GameObject player4 = Players.I.GameObjects[3];

            GameState.I.SetGameState(GameState.State.Running);

            //-Act
            yield return new WaitForSeconds(2);

            //-Assert
            Assert.Less(Mathf.Abs(Players.I.GetIdealX(player1) - player1.transform.position.x), 0.1f, "Player 1 has the correct position");
            Assert.Less(Mathf.Abs(Players.I.GetIdealX(player2) - player2.transform.position.x), 0.1f, "Player 2 has the correct position");
            Assert.Less(Mathf.Abs(Players.I.GetIdealX(player3) - player3.transform.position.x), 0.1f, "Player 3 has the correct position");
            Assert.Less(Mathf.Abs(Players.I.GetIdealX(player4) - player4.transform.position.x), 0.1f, "Player 4 has the correct position");
        }

        [UnityTest]
        public IEnumerator MoveToIdealPosition()
        {
            //-Arrange
            yield return Arrange.Reset();
            Arrange.SpawnPrefab("Test Prefabs/Scene");
            Arrange.SpawnPrefab("Test Prefabs/LevelGeneratorCoinsOnly");
            Players.I.AddPlayer(0, 0, "");
            Players.I.AddPlayer(1, 1, "");
            Players.I.AddPlayer(2, 2, "");
            Players.I.AddPlayer(3, 3, "");
            GameObject player1 = Players.I.GameObjects[0];
            GameObject player2 = Players.I.GameObjects[1];
            GameObject player3 = Players.I.GameObjects[2];
            GameObject player4 = Players.I.GameObjects[3];
            player1.transform.position = new Vector3(0, 0, 0);
            player2.transform.position = new Vector3(3, 0, 0);
            player3.transform.position = new Vector3(6, 0, 0);
            player4.transform.position = new Vector3(9, 0, 0);

            GameState.I.SetGameState(GameState.State.Running);

            //-Act
            yield return new WaitForSeconds(10);

            //-Assert
            Assert.Less(Mathf.Abs(Players.I.GetIdealX(player1) - player1.transform.position.x), 0.1f, "Player 1 has the correct position");
            Assert.Less(Mathf.Abs(Players.I.GetIdealX(player2) - player2.transform.position.x), 0.1f, "Player 2 has the correct position");
            Assert.Less(Mathf.Abs(Players.I.GetIdealX(player3) - player3.transform.position.x), 0.1f, "Player 3 has the correct position");
            Assert.Less(Mathf.Abs(Players.I.GetIdealX(player4) - player4.transform.position.x), 0.1f, "Player 4 has the correct position");
        }

        [UnityTest]
        public IEnumerator DontReachIdealPositionWithin1Seconds()
        {
            //-Arrange
            yield return Arrange.Reset();
            Arrange.SpawnPrefab("Test Prefabs/Scene");
            Arrange.SpawnPrefab("Test Prefabs/LevelGeneratorCoinsOnly");
            Players.I.AddPlayer(0, 0, "");
            Players.I.AddPlayer(1, 0, "");
            Players.I.AddPlayer(2, 0, "");
            Players.I.AddPlayer(3, 0, "");
            GameObject player1 = Players.I.GameObjects[0];
            GameObject player2 = Players.I.GameObjects[1];
            GameObject player3 = Players.I.GameObjects[2];
            GameObject player4 = Players.I.GameObjects[3];
            player1.transform.position = new Vector3(0, 0, 0);
            player2.transform.position = new Vector3(3, 0, 0);
            player3.transform.position = new Vector3(6, 0, 0);
            player4.transform.position = new Vector3(9, 0, 0);

            GameState.I.SetGameState(GameState.State.Running);

            //-Act
            //Wait for them to get to their ideal positions
            yield return new WaitForSeconds(1f);

            //-Assert
            Assert.AreNotEqual(Players.I.GetIdealX(player1), player1.transform.position.x, "Player 1 has the correct position");
            Assert.AreNotEqual(Players.I.GetIdealX(player2), player2.transform.position.x, "Player 2 has the correct position");
            Assert.AreNotEqual(Players.I.GetIdealX(player3), player3.transform.position.x, "Player 3 has the correct position");
            Assert.AreNotEqual(Players.I.GetIdealX(player4), player4.transform.position.x, "Player 4 has the correct position");
        }
    }
}
