using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.SceneManagement;

namespace AlienRunner.Tests
{
    public class Arrange

    {
        public static IEnumerator Reset()
        {
            Debug.Log("==Reset==");
            Debug.Log("Yielding arrange yield instruction...");
            yield return new ArrangeYieldInstruction();
            Debug.Log("Destroying Game Objects...");
            foreach (var obj in GameObject.FindObjectsOfType<MonoBehaviour>())
            {
                obj.StopAllCoroutines();
            }
            yield return null;
            foreach (var obj in GameObject.FindObjectsOfType<GameObject>())
            {
                GameObject.Destroy(obj);
            }
            yield return null;
            //TODO: Use a player prefs wrapper and reset it.
            Debug.Log("Setting up simulation speed...");
            SimulateSpeed.I.SimulatedFramerate = 30;
            SimulateSpeed.I.SimulationTargetFramerate = int.MaxValue;

            yield return null;
            Debug.Log("==Reset Done==");    
        }
        public static IEnumerator LoadScene(string sceneName)
        {
            SceneManager.LoadScene(sceneName);
            yield return new WaitForSecondsRealtime(1f);
        }
        /// <summary>
        /// Loads and returns a prefab with the specified path relative to the "Assets/Prefabs/" folder.
        /// </summary>
        public static GameObject LoadPrefab(string name)
        {
            GameObject prefab = null;
#if UNITY_EDITOR
            prefab = (GameObject)UnityEditor.AssetDatabase.LoadAssetAtPath("Assets/Prefabs/" + name + ".prefab", typeof(GameObject));
#endif
            if (prefab == null)
            {
                Debug.LogError("Cant find prefab " + name);
            }
            return prefab;
        }
        /// <summary>
        /// Finds and spawns the prefab with the specified path relative to the "Assets/Prefabs/" folder, then returns the instance.
        /// </summary>
        public static GameObject SpawnPrefab(string name)
        {
            GameObject prefab = LoadPrefab(name);
            GameObject spawned = Object.Instantiate(prefab, Vector3.zero, Quaternion.identity);
            spawned.name = prefab.name;
            return spawned;
        }
    }
}