using System.Collections;
using System.Collections.Generic;
using UnityEngine;

namespace AlienRunner.Tests
{
    public class SimulateSpeed : Singleton<SimulateSpeed>
    {
        /// <summary>
        /// The target framerate at which to simulate the game.
        /// </summary>
        public int SimulationTargetFramerate { get; set; }
        /// <summary>
        /// The framerate to use inside the simulation
        /// </summary>
        public int SimulatedFramerate { get; set; }
        private float simulatedDeltaTime => 1f / SimulatedFramerate;
        protected override bool InstantiateIfMissing => true;
        protected override bool UseDontDestroyOnLoad => true;

        private int originalTargetFramerate;
        private float originalTimescale;
        protected override void Awake()
        {
            this.originalTargetFramerate = Application.targetFrameRate;
            this.originalTimescale = Time.timeScale;
            base.Awake();
        }
        private void Update()
        {
            if(SimulatedFramerate > 0 && SimulationTargetFramerate > 0)
            {
                Application.targetFrameRate = SimulationTargetFramerate;
                Time.timeScale = simulatedDeltaTime / Time.unscaledDeltaTime;
            }
            else
            {
                Time.timeScale = 1f;   
            }
        }
        protected override void OnDestroy()
        {
            Time.timeScale = originalTimescale;
            Application.targetFrameRate = originalTargetFramerate;
            base.OnDestroy();
        }
    }
}