using System.Collections;
using System.Collections.Generic;
using UnityEngine;

namespace AlienRunner.Tests
{
    /// <summary>
    /// Contains static wrapper functions for a static eye instance, that looks in the entire scene.
    /// </summary>
    public class Eye : MonoBehaviour
    {
        /// <summary>
        /// The static eye instance for looking in the entire scene.
        /// </summary>
        private static EyeInstance eye = new EyeInstance(null);
        /// <summary>
        /// Use to look for GameObjects based on some text.
        /// </summary>
        public static EyeFunction<string> Text => eye.Text;
        /// <summary>
        /// Use to look for GameObjects based on a sprite name.
        /// </summary>
        public static EyeFunction<string> Sprite => eye.Sprite;
        /// <summary>
        /// Use to look for GameObjects based on a GameObject name.
        /// </summary>
        public static EyeFunction<string> Name => eye.Name;
        /// <summary>
        /// Use to look for GameObjects based on a text, sprite renderer or UI Image color.
        /// </summary>
        public static EyeFunction<Color> Color => eye.Color;
    }
}
